﻿using System.Collections.Generic;
using System.Text;
using System.Transactions;
using AncouApp.ChoBase.Core;
using Newtonsoft.Json;
using System;

namespace AncouApp.ChoBase
{
    public enum SortDirections { Asc, Desc }

    public static class ChoBase
    {

        #region Simple get / set methods


        /// <summary>
        /// This Is Property To Access ChoBase
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public static string GetS(string keyBase, params object[] args)
        {
            string rtn = KvsAccesser.SelectByKey(GetKey(keyBase, args));
            return rtn;
        }

        /// <summary>
        /// set string value by key.
        /// </summary>
        /// <param name="keyBase"></param>
        /// <param name="val"></param>
        /// <param name="args"></param>
        public static void SetS(string keyBase, string val, params object[] args)
        {
            KvsAccesser.Merge(GetKey(keyBase, args), val);
        }
        /// <summary>
        /// Get value as int32
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="key"></param>
        /// <returns></returns>
        public static int GetI(string key, params object[] args)
        {
            string str = GetS(key, args);
			int val = Convert.ToInt32(str) ;
            return val;
        }        
		
		/// <summary>
        /// Set 
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="key"></param>
        /// <param name="val"></param>
        public static void SetI(string key, int val, params object[] args)
        {
            SetS(key, val.ToString(), args);
        }
		
        /// <summary>
        /// Get Value with Json Deserialize.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="key"></param>
        /// <returns></returns>
        public static T GetJ<T>(string key, params object[] args)
        {
            string jsonString = GetS(key, args);
            T val = JsonConvert.DeserializeObject<T>(jsonString);
            return val;

        }        
		
		/// <summary>
        /// Set Value the objetc about JSON.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="key"></param>
        /// <param name="val"></param>
        public static void SetJ<T>(string key, T val, params object[] args)
        {
            string jsonString = JsonConvert.SerializeObject(val);
            SetS(key, jsonString, args);
        }


        #endregion
		
        /// <summary>
        /// Prefix Search By Key
        /// </summary>
        /// <param name="key">key</param>
        /// <param name="direction">Key Sort Direction</param>
        /// <returns>Values</returns>
        public static IList<string> ScanByKey(string key, SortDirections direction)
        {
            IList<string> rtn = KvsAccesser.ScanByKey(key, direction);
            return rtn;
        }

        #region Delete Methods

        /// <summary>
        /// Delete By Key
        /// </summary>
        /// <param name="key">Key</param>
        /// <returns>Count of deleted</returns>
        public static int Remove(string keyBase, params object[] args)
        {
            int rtn = KvsAccesser.DeleteByKey(GetKey(keyBase, args));
            return rtn;
        }

        /// <summary>
        /// Delete by any keys.
        /// </summary>
        /// <param name="keys">keys</param>
        /// <returns>deleted count</returns>
        public static int RemoveByKeys(params string[] keys)
        {
            int count = 0;
            using (TransactionScope scope = new TransactionScope())
            {
                foreach (string key in keys) count += Remove(key);
            }
            return count;
        }

        #endregion

        #region CheckMethods
        /// <summary>
        /// Check is exist key in Kavos
        /// </summary>
        /// <param name="key">check key</param>
        /// <returns></returns>
        public static bool ContainsKey(string key, params object[] args)
        {
            try
            {
                GetS(GetKey(key, args));
                return true;
            }
            catch (KeyNotFoundException)
            {
                return false;
            }
        }
        #endregion


        private static string GetKey(string keyBase, params object[] args)
        {
            if (args == null || args.Length == 0) { return keyBase; }
            StringBuilder rtn = new StringBuilder(keyBase);
            foreach (var arg in args)
            {
                rtn.Append("." + arg);
            }
            return rtn.ToString();
        }
    }
}
